<?xml version="1.0" encoding="UTF-8"?>

<!--*********************************************************************************************************** 
	created by: Laura Robinson
	contact: lrobinson@secstate.wa.gov
	create date: 2009-05-01
	notes: this file was written and tested with the SAXON 6.5.3 transformation processor and is meant to 
	tranform the input file (e.g. issueTbl.xml) to recreate the appropriate database table while comparing it 
	to the output from the batch files (batch.xml and mets issue files). It recreates the table with 
	color-coded backgrounds indicating possible missing or mismatched data. 

	to do:
	1. update keys for issueTbl
	2. update exceptions in cellMatch template for those fields not tested
	3. update exceptions in metsIssueMatch template for special testing
	***********************************************************************************************************-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns:mods="http://www.loc.gov/mods/v3" xmlns:ndnp="http://www.loc.gov/ndnp" xmlns:mets="http://www.loc.gov/METS/" version="1.1">
	<xsl:output method="html"/>
	<!--*******************************************************************************************************
		param: passed in from command line via the run_report.bat file
		*******************************************************************************************************-->
	<xsl:param name="batchFilePath"/>
	<!--*******************************************************************************************************
		declare keys: used to match test output for missing data from issueTbl
		*******************************************************************************************************-->
	<!--batchIssues key used to make sure batch includes the issue record first-->
	<xsl:key name="batchIssues" match="ndnp:batch/ndnp:issue" use="concat(@lccn, @issueDate, @editionOrder)"/>
	<!--*****************************************************
		NOTE: keys below double as field names for issueTbl 
		WARNING: don't alter or add to @name attributes
		*****************************************************-->
	<!--may have to change Title key to a contains-->
	<xsl:key name="Title" match="*/@LABEL" use="substring(*/@LABEL, 1, string-length(*/@LABEL) - 12)"/>
	<!--lccn not represented in mets reel - directory name checked instead during titleTbl test-->
	<xsl:key name="lccn" match="//mods:identifier[@type='lccn']" use="."/>
	<xsl:key name="ReelNumber" match="//mods:identifier[@type='reel number']" use="."/>
	<!--ndnp:startDate is really first date of title (not reel) - not a true test for multiple title reels-->
	<xsl:key name="VolNumber" match="//mods:detail[@type='volume']/mods:number" use="."/>
	<!--ndnp:endDate is really last date of title (not reel) - not a true test for multiple title reels-->
	<xsl:key name="IssueNumber" match="//mods:detail[@type='issue']/mods:number" use="."/>
	<xsl:key name="EditionNumber" match="//mods:detail[@type='edition']/mods:number" use="."/>
	<xsl:key name="EditionLabel" match="//mods:detail[@type='edition']/mods:caption" use="."/>
	<xsl:key name="IssueDate" match="//mods:dateIssued[1]" use="."/>
	<xsl:key name="DateAsPrinted" match="//mods:dateIssued[@qualifier='questionable']" use="."/>
	<xsl:key name="IssuePresent" match="//mods:note[@type='noteAboutReproduction']" use="."/>
	<xsl:key name="PhysDescription" match="//mods:form/@type" use="."/>
	<!--questionable key match - needs more testing-->
	<xsl:key name="SectionCount" match="//mods:detail[@type='section label'][position() = last()]" use="position()"/>
	<xsl:key name="SectionLabels" match="//mods:detail[@type='section label']/mods:number" use="."/>
	<!--questionable key match - needs more testing-->
	<xsl:key name="IssuePageCount" match="//mets:dmdSec[position() = last()]//mods:extent[@unit='pages']/mods:start" use="."/>
	<xsl:key name="PagesMissing" match="PagesMissing" use="."/>
	<xsl:key name="DuplicatePages" match="DuplicatePages" use="."/>
	<xsl:key name="Notes" match="Notes" use="."/>
	<!--*******************************************************************************************************
		declare global variables: used to gain access to other documents or set constants throughout xsl file
		*******************************************************************************************************-->
	<xsl:variable name="batchFile" select="document($batchFilePath)"/>
	<xsl:variable name="thisFile" select="document('issueTbl.xsl')"/>
	<xsl:template match="dataroot">
		<!--*******************************************************************************************************
		test page header and body
		*******************************************************************************************************-->
		<html>
			<body>
				<h1>Test: issueTbl</h1>
				<!-- to gain access to root element attributes use * rather than . -->
				<h3>Batch: <xsl:value-of select="$batchFile/*/@name"/></h3>
				<!--*******testing ... issue list perhaps too long for report view -->
				<!-- Ordered list of issues in batch.xml file 
				<h3>List of issues in batch:</h3>
				<ol>
					<xsl:for-each select="$batchFile/ndnp:batch/ndnp:issue">
						<li>
							<xsl:value-of select="concat(@lccn, '.', @issueDate, '.', @editionOrder)"/>
						</li>
					</xsl:for-each>
				</ol>-->
				<xsl:call-template name="table"/>
				<xsl:element name="a">
					<xsl:attribute name="href">titleTbl_report.html</xsl:attribute>
					<xsl:text>title_report.html</xsl:text>
				</xsl:element>
				<br/>
				<xsl:element name="a">
					<xsl:attribute name="href">reelTbl_report.html</xsl:attribute>
					<xsl:text>reelTbl_report.html</xsl:text>
				</xsl:element>
			</body>
		</html>
	</xsl:template>
	<!--*******************************************************************************************************
		table header and header row
		*******************************************************************************************************-->
	<xsl:template name="table">
		<h3>issueTbl:</h3>
		<xsl:text>Note: Titles in red are missing from</xsl:text>
		<a>
			<xsl:attribute name="href">
				<xsl:value-of select="$batchFilePath"/>
			</xsl:attribute>
			<xsl:attribute name="target">
				<xsl:text>_blank</xsl:text>
			</xsl:attribute>
			<xsl:value-of select="$batchFilePath"/>
		</a>
		<br/>
		<xsl:text>Note: Cells in yellow may not be wrong but require further investigation.</xsl:text>
		<table border="1">
			<!--table header-->
			<tr>
				<!--navigates within this xsl file to the variables below to read all possible names and create field header-->
				<xsl:for-each select="$thisFile/xsl:stylesheet/xsl:key[@name != 'batchIssues']">
					<td>
						<xsl:value-of select="@name"/>
					</td>
				</xsl:for-each>
			</tr>
			<xsl:call-template name="rows"/>
		</table>
	</xsl:template>
	<!--*******************************************************************************************************
		rows recreated from all issueTbl.xml issueTbl elements
		*******************************************************************************************************-->
	<xsl:template name="rows">
		<xsl:for-each select="issueTbl">
			<!--select the whole issueTbl record-->
			<xsl:variable name="issueTblRec" select="."/>
			<!--tests to see if there is a batch.xml ndnp:issue match-->
			<xsl:variable name="issueMatch">
				<!--must first get into the context of Batch.xml in order to test the value -->
				<xsl:for-each select="$batchFile">
					<!--key() function returns the matching node. Therefore, apply-templates checks for value-->
					<xsl:apply-templates select="key('batchIssues', concat($issueTblRec/lccn, $issueTblRec/IssueDate, $issueTblRec/EditionNumber))"/>
				</xsl:for-each>
			</xsl:variable>
			<tr>
				<!--test to see if issue exists in batch.xml, if not turns the whole row red-->
				<xsl:choose>
					<xsl:when test="$issueMatch = ''">
						<xsl:call-template name="bgColorTest">
							<xsl:with-param name="color" select="'red'"/>
						</xsl:call-template>
						<!--build cells within row here using issueTbl records that don't have matching issue mets files-->
						<xsl:for-each select="$thisFile/xsl:stylesheet/xsl:key[@name != 'batchIssues']">
							<xsl:variable name="fieldName" select="@name"/>
							<td>
								<xsl:value-of select="$issueTblRec/*[name() = $fieldName]"/>
							</td>
						</xsl:for-each>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="cellMatch">
							<xsl:with-param name="issueTblRec" select="$issueTblRec"/>
							<xsl:with-param name="metsIssuePath" select="$issueMatch"/>
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</tr>
		</xsl:for-each>
	</xsl:template>
	<!--*******************************************************************************************************
		cell values recreated from issueTbl.xml issueTbl/* elements
		*******************************************************************************************************-->
	<xsl:template name="cellMatch">
		<xsl:param name="issueTblRec"/>
		<xsl:param name="metsIssuePath"/>
		<!--automatically create a cell for each possible field name again by navigating to variables above and then input what comes from that variable-->
		<xsl:for-each select="$thisFile/xsl:stylesheet/xsl:key[@name != 'batchIssues']">
			<xsl:variable name="fieldName" select="@name"/>
			<xsl:variable name="fieldMatch" select="@match"/>
			<td>
				<xsl:choose>
					<xsl:when test="$fieldName[. != 'PagesMissing'][. != 'DuplicatePages'][. != 'Notes'] and $issueTblRec/*[name() = $fieldName] != ''">
						<xsl:call-template name="metsIssueMatch">
							<xsl:with-param name="docPath" select="$metsIssuePath"/>
							<xsl:with-param name="keyName" select="$fieldName"/>
							<xsl:with-param name="keyMatch" select="$fieldMatch"/>
							<xsl:with-param name="cellValue" select="$issueTblRec/*[name() = $fieldName]"/>
							<xsl:with-param name="extraPages">
								<xsl:if test="$fieldName = 'IssuePageCount'">
								<xsl:variable name="dups">
									<xsl:call-template name="countCommas">
										<xsl:with-param name="string" select="$issueTblRec/DuplicatePages"/>
										<xsl:with-param name="substr" select="','"/>
									</xsl:call-template>
								</xsl:variable>
								<xsl:variable name="missing">
									<xsl:call-template name="countCommas">
										<xsl:with-param name="string" select="$issueTblRec/PagesMissing"/>
										<xsl:with-param name="substr" select="','"/>
									</xsl:call-template>
								</xsl:variable>
								<xsl:value-of select="$dups - $missing"/>
								</xsl:if>
							</xsl:with-param>
						</xsl:call-template>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$issueTblRec/*[name() = $fieldName]"/>
					</xsl:otherwise>
				</xsl:choose>
			</td>
		</xsl:for-each>
	</xsl:template>
	<!--*******************************************************************************************************
		cell test using paths from Batch.xml to mets issue files
		*******************************************************************************************************-->
	<xsl:template name="metsIssueMatch">
		<xsl:param name="docPath"/>
		<xsl:param name="keyName"/>
		<xsl:param name="keyMatch"/>
		<xsl:param name="cellValue"/>
		<xsl:param name="extraPages"/>
		<!--need to be recursive so we have to test for and take string before '.xml' then add it back in-->
		<xsl:variable name="pathParse" select="concat(substring-before($batchFilePath, 'BATCH.xml'), substring-before(substring-after($docPath, './'), '.xml'), '.xml')"/>
		<xsl:variable name="metsFile" select="document($pathParse)"/>
		<xsl:variable name="leftOvers" select="substring-after($docPath, '.xml')"/>
		<xsl:variable name="metsMatch">
			<xsl:for-each select="$metsFile">
				<!--***********************************************************************************************
					WARNING: key() only checks the first occurance - need to investigate for-each loop possiblities 
					***********************************************************************************************-->
				<xsl:choose>
					<xsl:when test="$keyName = 'Title'">
						<xsl:if test="substring(*/@LABEL, 1, string-length(*/@LABEL) - 12) != $cellValue">
							<xsl:text>mismatch</xsl:text>
						</xsl:if>
					</xsl:when>
					<!--don't run tests on certain elements if issue is no 'Present'-->
					<xsl:when test="not(key('IssuePresent', 'Present')) and $keyName = 'ReelNumber' or $keyName = 'VolNumber' or $keyName = 'IssueNumber' or $keyName = 'PhysDescription'"/>
					<xsl:when test="$keyName = 'IssuePresent'">
						<xsl:choose>
							<xsl:when test="$cellValue = '1'">
								<xsl:if test="not(key($keyName, 'Present'))">
									<xsl:text>mismatch</xsl:text>
								</xsl:if>
							</xsl:when>
							<xsl:when test="$cellValue = '0'">
								<xsl:if test="not(key($keyName, 'Not digitized, published')) and not(key($keyName, 'Not digitized, not published')) and not(key($keyName, 'Not digitized, publishing unknown'))">
									<xsl:text>mismatch</xsl:text>
								</xsl:if>
							</xsl:when>
							<xsl:otherwise>
								<xsl:if test="not(key($keyName, $cellValue))">
									<xsl:text>mismatch</xsl:text>
								</xsl:if>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:when test="$keyName = 'IssuePageCount'">
						<xsl:if test="not(key($keyName, $cellValue + $extraPages))">
							<xsl:text>mismatch</xsl:text>
						</xsl:if>					
					</xsl:when>					
					<xsl:when test="not(key($keyName, $cellValue))">
						<xsl:text>mismatch</xsl:text>
					</xsl:when>
				</xsl:choose>
			</xsl:for-each>
		</xsl:variable>
		<xsl:choose>
			<!--when there is an error, there will be a value in $metsMatch-->
			<xsl:when test="$metsMatch = 'mismatch'">
				<xsl:call-template name="bgColorTest">
					<xsl:with-param name="color" select="'yellow'"/>
				</xsl:call-template>
				<xsl:value-of select="$cellValue"/>
				<br/>
				<xsl:call-template name="metsLink">
					<xsl:with-param name="link" select="$pathParse"/>
					<xsl:with-param name="elementPath" select="$keyMatch"/>
				</xsl:call-template>
			</xsl:when>
			<!--when there is no error but more mets-reel documents call template again with $leftOvers-->
			<xsl:when test="$metsMatch = '' and $leftOvers != ''">
				<xsl:call-template name="metsIssueMatch">
					<xsl:with-param name="docPath" select="$leftOvers"/>
					<xsl:with-param name="keyName" select="$keyName"/>
					<xsl:with-param name="keyMatch" select="$keyMatch"/>
					<xsl:with-param name="cellValue" select="$cellValue"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$cellValue"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!--*******************************************************************************************************
		sets the link to the cellValue
		*******************************************************************************************************-->
	<xsl:template name="metsLink">
		<xsl:param name="link"/>
		<xsl:param name="elementPath"/>
		<xsl:text>[mismatch in: </xsl:text>
		<xsl:element name="a">
			<xsl:attribute name="href">
				<xsl:value-of select="$link"/>
			</xsl:attribute>
			<xsl:attribute name="target">
				<xsl:text>_blank</xsl:text>
			</xsl:attribute>
			<xsl:value-of select="$elementPath"/>
		</xsl:element>
		<xsl:text>]</xsl:text>
	</xsl:template>
	<!--*******************************************************************************************************
		sets the background color based on rules (match, mismatched, or missing)
		*******************************************************************************************************-->
	<xsl:template name="bgColorTest">
		<xsl:param name="color"/>
		<xsl:attribute name="bgcolor">
			<xsl:value-of select="$color"/>
		</xsl:attribute>
	</xsl:template>
	<!--*******************************************************************************************************
		gets a number of pages by counting the commas
		*******************************************************************************************************-->
	<xsl:template name="countCommas">
		<xsl:param name="string"/>
		<xsl:param name="substr"/>
		<xsl:choose>
			<xsl:when test="contains($string, $substr) and $string and $substr">
				<xsl:variable name="rest">
					<xsl:call-template name="countCommas">
						<xsl:with-param name="string" select="substring-after($string, $substr)"/>
						<xsl:with-param name="substr" select="$substr"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:value-of select="$rest + 1"/>
			</xsl:when>
			<xsl:when test="not(contains($string, $substr)) and $string != ''">1</xsl:when>			
			<xsl:otherwise>0</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
</xsl:stylesheet>